/*******************************************************************************
* File Name: LCDChar.h
* Version 2.20
*
* Description:
*  This header file contains registers and constants associated with the
*  Character LCD component.
*
* Note:
*
********************************************************************************
* Copyright 2008-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#if !defined(CY_CHARLCD_LCDChar_H)
#define CY_CHARLCD_LCDChar_H

#include "cytypes.h"
#include "cyfitter.h"


/***************************************
*   Conditional Compilation Parameters
***************************************/

#define LCDChar_CONVERSION_ROUTINES     (1u)
#define LCDChar_CUSTOM_CHAR_SET         (0u)

/* Custom character set types */
#define LCDChar_NONE                     (0u)    /* No Custom Fonts      */
#define LCDChar_HORIZONTAL_BG            (1u)    /* Horizontal Bar Graph */
#define LCDChar_VERTICAL_BG              (2u)    /* Vertical Bar Graph   */
#define LCDChar_USER_DEFINED             (3u)    /* User Defined Fonts   */


/***************************************
*     Data Struct Definitions
***************************************/

/* Sleep Mode API Support */
typedef struct
{
    uint8 enableState;
} LCDChar_BACKUP_STRUCT;


/***************************************
*        Function Prototypes
***************************************/

void LCDChar_Init(void) ;
void LCDChar_Enable(void) ;
void LCDChar_Start(void) ;
void LCDChar_Stop(void) ;
void LCDChar_WriteControl(uint8 cByte) ;
void LCDChar_WriteData(uint8 dByte) ;
void LCDChar_PrintString(char8 const string[]) ;
void LCDChar_Position(uint8 row, uint8 column) ;
void LCDChar_PutChar(char8 character) ;
void LCDChar_IsReady(void) ;
void LCDChar_SaveConfig(void) ;
void LCDChar_RestoreConfig(void) ;
void LCDChar_Sleep(void) ;
void LCDChar_Wakeup(void) ;

#if((LCDChar_CUSTOM_CHAR_SET == LCDChar_VERTICAL_BG) || \
                (LCDChar_CUSTOM_CHAR_SET == LCDChar_HORIZONTAL_BG))

    void  LCDChar_LoadCustomFonts(uint8 const customData[])
                        ;

    void  LCDChar_DrawHorizontalBG(uint8 row, uint8 column, uint8 maxCharacters, uint8 value)
                         ;

    void LCDChar_DrawVerticalBG(uint8 row, uint8 column, uint8 maxCharacters, uint8 value)
                        ;

#endif /* ((LCDChar_CUSTOM_CHAR_SET == LCDChar_VERTICAL_BG) */

#if(LCDChar_CUSTOM_CHAR_SET == LCDChar_USER_DEFINED)

    void LCDChar_LoadCustomFonts(uint8 const customData[])
                            ;

#endif /* ((LCDChar_CUSTOM_CHAR_SET == LCDChar_USER_DEFINED) */

#if(LCDChar_CONVERSION_ROUTINES == 1u)

    /* ASCII Conversion Routines */
    void LCDChar_PrintInt8(uint8 value) ;
    void LCDChar_PrintInt16(uint16 value) ;
    void LCDChar_PrintInt32(uint32 value) ;
    void LCDChar_PrintNumber(uint16 value) ; 
    void LCDChar_PrintU32Number(uint32 value) ;
    
#endif /* LCDChar_CONVERSION_ROUTINES == 1u */

/* Clear Macro */
#define LCDChar_ClearDisplay() LCDChar_WriteControl(LCDChar_CLEAR_DISPLAY)

/* Off Macro */
#define LCDChar_DisplayOff() LCDChar_WriteControl(LCDChar_DISPLAY_CURSOR_OFF)

/* On Macro */
#define LCDChar_DisplayOn() LCDChar_WriteControl(LCDChar_DISPLAY_ON_CURSOR_OFF)

#define LCDChar_PrintNumber(value) LCDChar_PrintU32Number((uint16) (value))


/***************************************
*           Global Variables
***************************************/

extern uint8 LCDChar_initVar;
extern uint8 LCDChar_enableState;
extern uint8 const CYCODE LCDChar_customFonts[64u];


/***************************************
*           API Constants
***************************************/

/* Full Byte Commands Sent as Two Nibbles */
#define LCDChar_DISPLAY_8_BIT_INIT       (0x03u)
#define LCDChar_DISPLAY_4_BIT_INIT       (0x02u)
#define LCDChar_DISPLAY_CURSOR_OFF       (0x08u)
#define LCDChar_CLEAR_DISPLAY            (0x01u)
#define LCDChar_CURSOR_AUTO_INCR_ON      (0x06u)
#define LCDChar_DISPLAY_CURSOR_ON        (0x0Eu)
#define LCDChar_DISPLAY_2_LINES_5x10     (0x2Cu)
#define LCDChar_DISPLAY_ON_CURSOR_OFF    (0x0Cu)

#define LCDChar_RESET_CURSOR_POSITION    (0x03u)
#define LCDChar_CURSOR_WINK              (0x0Du)
#define LCDChar_CURSOR_BLINK             (0x0Fu)
#define LCDChar_CURSOR_SH_LEFT           (0x10u)
#define LCDChar_CURSOR_SH_RIGHT          (0x14u)
#define LCDChar_DISPLAY_SCRL_LEFT        (0x18u)
#define LCDChar_DISPLAY_SCRL_RIGHT       (0x1Eu)
#define LCDChar_CURSOR_HOME              (0x02u)
#define LCDChar_CURSOR_LEFT              (0x04u)
#define LCDChar_CURSOR_RIGHT             (0x06u)

/* Point to Character Generator Ram 0 */
#define LCDChar_CGRAM_0                  (0x40u)

/* Point to Display Data Ram 0 */
#define LCDChar_DDRAM_0                  (0x80u)

/* LCD Characteristics */
#define LCDChar_CHARACTER_WIDTH          (0x05u)
#define LCDChar_CHARACTER_HEIGHT         (0x08u)

#if(LCDChar_CONVERSION_ROUTINES == 1u)
    #define LCDChar_NUMBER_OF_REMAINDERS_U32 (0x0Au)
    #define LCDChar_TEN                      (0x0Au)
    #define LCDChar_8_BIT_SHIFT              (8u)
    #define LCDChar_32_BIT_SHIFT             (32u)
    #define LCDChar_ZERO_CHAR_ASCII          (48u)
#endif /* LCDChar_CONVERSION_ROUTINES == 1u */

/* Nibble Offset and Mask */
#define LCDChar_NIBBLE_SHIFT             (0x04u)
#define LCDChar_NIBBLE_MASK              (0x0Fu)

/* LCD Module Address Constants */
#define LCDChar_ROW_0_START              (0x80u)
#define LCDChar_ROW_1_START              (0xC0u)
#define LCDChar_ROW_2_START              (0x94u)
#define LCDChar_ROW_3_START              (0xD4u)

/* Custom Character References */
#define LCDChar_CUSTOM_0                 (0x00u)
#define LCDChar_CUSTOM_1                 (0x01u)
#define LCDChar_CUSTOM_2                 (0x02u)
#define LCDChar_CUSTOM_3                 (0x03u)
#define LCDChar_CUSTOM_4                 (0x04u)
#define LCDChar_CUSTOM_5                 (0x05u)
#define LCDChar_CUSTOM_6                 (0x06u)
#define LCDChar_CUSTOM_7                 (0x07u)

/* Other constants */
#define LCDChar_BYTE_UPPER_NIBBLE_SHIFT  (0x04u)
#define LCDChar_BYTE_LOWER_NIBBLE_MASK   (0x0Fu)
#define LCDChar_U16_UPPER_BYTE_SHIFT     (0x08u)
#define LCDChar_U16_LOWER_BYTE_MASK      (0xFFu)
#define LCDChar_CUSTOM_CHAR_SET_LEN      (0x40u)

#define LCDChar_LONGEST_CMD_US           (0x651u)
#define LCDChar_WAIT_CYCLE               (0x10u)
#define LCDChar_READY_DELAY              ((LCDChar_LONGEST_CMD_US * 4u)/(LCDChar_WAIT_CYCLE))


/***************************************
*             Registers
***************************************/

/* Device specific registers */
#if (CY_PSOC4)

    #define LCDChar_PORT_DR_REG           (*(reg32 *) LCDChar_LCDPort__DR)  /* Data Output Register */
    #define LCDChar_PORT_DR_PTR           ( (reg32 *) LCDChar_LCDPort__DR)
    #define LCDChar_PORT_PS_REG           (*(reg32 *) LCDChar_LCDPort__PS)  /* Pin State Register */
    #define LCDChar_PORT_PS_PTR           ( (reg32 *) LCDChar_LCDPort__PS)
    
    #define LCDChar_PORT_PC_REG           (*(reg32 *) LCDChar_LCDPort__PC)
    #define LCDChar_PORT_PC_PTR           (*(reg32 *) LCDChar_LCDPort__PC)
    
#else

    #define LCDChar_PORT_DR_REG           (*(reg8 *) LCDChar_LCDPort__DR)  /* Data Output Register */
    #define LCDChar_PORT_DR_PTR           ( (reg8 *) LCDChar_LCDPort__DR)
    #define LCDChar_PORT_PS_REG           (*(reg8 *) LCDChar_LCDPort__PS)  /* Pin State Register */
    #define LCDChar_PORT_PS_PTR           ( (reg8 *) LCDChar_LCDPort__PS)

    #define LCDChar_PORT_DM0_REG          (*(reg8 *) LCDChar_LCDPort__DM0) /* Port Drive Mode 0 */
    #define LCDChar_PORT_DM0_PTR          ( (reg8 *) LCDChar_LCDPort__DM0)
    #define LCDChar_PORT_DM1_REG          (*(reg8 *) LCDChar_LCDPort__DM1) /* Port Drive Mode 1 */
    #define LCDChar_PORT_DM1_PTR          ( (reg8 *) LCDChar_LCDPort__DM1)
    #define LCDChar_PORT_DM2_REG          (*(reg8 *) LCDChar_LCDPort__DM2) /* Port Drive Mode 2 */
    #define LCDChar_PORT_DM2_PTR          ( (reg8 *) LCDChar_LCDPort__DM2)

#endif /* CY_PSOC4 */


/***************************************
*       Register Constants
***************************************/

/* SHIFT must be 1 or 0 */
#if (0 == LCDChar_LCDPort__SHIFT)
    #define LCDChar_PORT_SHIFT               (0x00u)
#else
    #define LCDChar_PORT_SHIFT               (0x01u)
#endif /* (0 == LCDChar_LCDPort__SHIFT) */

#define LCDChar_PORT_MASK                ((uint8) (LCDChar_LCDPort__MASK))

#if (CY_PSOC4)

    #define LCDChar_DM_PIN_STEP              (3u)
    /* Hi-Z Digital is defined by the value of "001b" and this should be set for
    * four data pins, in this way we get - 0x00000249. A similar Strong drive
    * is defined with "110b" so we get 0x00000DB6.
    */
    #define LCDChar_HIGH_Z_DATA_DM           ((0x00000249ul) << (LCDChar_PORT_SHIFT *\
                                                                          LCDChar_DM_PIN_STEP))
    #define LCDChar_STRONG_DATA_DM           ((0x00000DB6ul) << (LCDChar_PORT_SHIFT *\
                                                                          LCDChar_DM_PIN_STEP))
    #define LCDChar_DATA_PINS_MASK           (0x00000FFFul)
    #define LCDChar_DM_DATA_MASK             ((uint32) (LCDChar_DATA_PINS_MASK << \
                                                      (LCDChar_PORT_SHIFT * LCDChar_DM_PIN_STEP)))

#else

    /* Drive Mode register values for High Z */
    #define LCDChar_HIGH_Z_DM0               (0xFFu)
    #define LCDChar_HIGH_Z_DM1               (0x00u)
    #define LCDChar_HIGH_Z_DM2               (0x00u)

    /* Drive Mode register values for High Z Analog */
    #define LCDChar_HIGH_Z_A_DM0             (0x00u)
    #define LCDChar_HIGH_Z_A_DM1             (0x00u)
    #define LCDChar_HIGH_Z_A_DM2             (0x00u)

    /* Drive Mode register values for Strong */
    #define LCDChar_STRONG_DM0               (0x00u)
    #define LCDChar_STRONG_DM1               (0xFFu)
    #define LCDChar_STRONG_DM2               (0xFFu)

#endif /* CY_PSOC4 */

/* Pin Masks */
#define LCDChar_RS                     ((uint8) \
                                                (((uint8) 0x20u) << LCDChar_LCDPort__SHIFT))
#define LCDChar_RW                     ((uint8) \
                                                (((uint8) 0x40u) << LCDChar_LCDPort__SHIFT))
#define LCDChar_E                      ((uint8) \
                                                (((uint8) 0x10u) << LCDChar_LCDPort__SHIFT))
#define LCDChar_READY_BIT              ((uint8) \
                                                (((uint8) 0x08u) << LCDChar_LCDPort__SHIFT))
#define LCDChar_DATA_MASK              ((uint8) \
                                                (((uint8) 0x0Fu) << LCDChar_LCDPort__SHIFT))

/* These names are obsolete and will be removed in future revisions */
#define LCDChar_PORT_DR                  LCDChar_PORT_DR_REG
#define LCDChar_PORT_PS                  LCDChar_PORT_PS_REG
#define LCDChar_PORT_DM0                 LCDChar_PORT_DM0_REG
#define LCDChar_PORT_DM1                 LCDChar_PORT_DM1_REG
#define LCDChar_PORT_DM2                 LCDChar_PORT_DM2_REG


/***************************************
*       Obsolete function names
***************************************/
#if(LCDChar_CONVERSION_ROUTINES == 1u)
    /* This function names are obsolete and will be removed in future 
    * revisions of the component.
    */
    #define LCDChar_PrintDecUint16(x)   LCDChar_PrintNumber(x)  
    #define LCDChar_PrintHexUint8(x)    LCDChar_PrintInt8(x)
    #define LCDChar_PrintHexUint16(x)   LCDChar_PrintInt16(x)        

#endif /* LCDChar_CONVERSION_ROUTINES == 1u */

#endif /* CY_CHARLCD_LCDChar_H */


/* [] END OF FILE */
